import {
  IsUUID,
  Length,
  Validate,
  IsString,
  IsNotEmpty,
  IsNumber,
  Min,
  Max,
  IsOptional,
  IsHexColor,
  IsIn,
} from 'class-validator';

import { GameEntity } from '../game/game.entity';
import { RoleValidation, Uuid } from '../shared/custom-validation';
import { GameDTO } from '../game/game.dto';
import { FactionEntity, GameGroupEntity } from './faction.entity';

export class FactionDTO {
  @IsOptional()
  @IsUUID('4')
  factionId?: string;
  @IsString()
  @IsNotEmpty()
  @Length(2, 31)
  factionName: string;
  @IsHexColor()
  colour: string;
  @IsString()
  @IsNotEmpty()
  @Length(3, 15)
  factionPassword: string;
  @IsNumber()
  @Min(1)
  @Max(3)
  multiplier?: number;
  game: GameDTO;
}

export class JoinFactionDTO {
  @IsUUID('4')
  factionId: string;
  @Length(3, 31)
  factionPassword: string;
  @IsUUID('4')
  game: GameEntity;
}

export class PromotePlayerDTO {
  @IsUUID('4')
  player: string;
  @Validate(RoleValidation)
  role: string;
}

export class GameGroupDTO {
  @IsString()
  @Length(3, 31)
  name: string;
  @IsIn(['infantry', 'recon', 'mechanized'])
  class: string;
  @IsUUID('4')
  faction: FactionEntity;
}

export class JoinGameGroupDTO {
  @IsUUID('4')
  groupId: GameGroupEntity;
}
