import {
  IsString,
  Length,
  IsBoolean,
  Validate,
  IsUUID,
  Equals,
} from 'class-validator';
import { FactionEntity } from '../faction/faction.entity';
import { GameEntity } from '../game/game.entity';
import { Uuid } from '../shared/custom-validation';

export class CreateTaskDTO {
  @IsString()
  @Length(3, 31)
  taskName: string;
  @IsString()
  @Length(0, 255)
  taskDescription: string;
  @IsBoolean()
  taskIsActive: boolean;
  @Validate(Uuid)
  faction: FactionEntity;
  @Equals(null)
  taskWinner: FactionEntity;
  // faction unique id
  @IsUUID('4')
  taskGame: GameEntity;
}

export class EditTaskDTO {
  @IsUUID('4')
  taskId: string;
  @IsUUID('4')
  taskWinner: FactionEntity;
  @IsUUID('4')
  taskGame: GameEntity;
}

export class DeleteTaskDTO {
  @IsUUID('4')
  taskId: string;
}
