import {
  Entity,
  Column,
  PrimaryGeneratedColumn,
  BeforeInsert,
  OneToMany,
} from 'typeorm';
import * as bcrypt from 'bcryptjs';
import * as jwt from 'jsonwebtoken';

import { Game_PersonEntity } from '../game/game.entity';
import { Exclude } from 'class-transformer';

@Entity('Person')
export class PersonEntity {
  @PrimaryGeneratedColumn('uuid') id: string;
  @Column({ type: 'text', unique: true }) name: string;

  @Exclude()
  @Column('text')
  password: string;

  @OneToMany(type => Game_PersonEntity, game_persons => game_persons.person)
  game_persons: Game_PersonEntity[];

  // hashes the password before inserting it to database
  @BeforeInsert()
  async hashPassword() {
    this.password = await bcrypt.hash(this.password, 10);
  }

  // returns username and associated token
  tokenObject() {
    const { name, token } = this;
    return { name, token };
  }

  async comparePassword(attempt: string) {
    return await bcrypt.compareSync(attempt, this.password);
  }

  private get token() {
    const { id, name } = this;
    return jwt.sign(
      {
        id,
        name,
      },
      process.env.SECRET,
      { expiresIn: '7d' },
    );
  }
}
